@@ -20,7 +20,6 @@ match:
 		const void *ip,
 		const struct xt_match *match,
 		void *matchinfo,
-		unsigned int matchinfosize,
 		unsigned int hook_mask,
 	);
 
@@ -28,7 +27,6 @@ match:
 	(*destroy)(
 		const struct xt_match *match,
 		void *matchinfo,
-		unsigned int matchinfosize,
 	);
 
 target:
@@ -42,7 +40,6 @@ target:
 		unsigned int hooknum,
 		const struct xt_target *target,
 		const void *targinfo,
-		void *userdata,
 	);
 
 	/* error code */
@@ -52,7 +49,6 @@ target:
 		const void *entry,
 		const struct xt_target *target,
 		void *targinfo,
-		unsigned int targinfosize,
 		unsigned int hook_mask,
 	);
 
@@ -60,5 +56,4 @@ target:
 	(*destroy)(
 		const struct xt_target *target,
 		void *targinfo,
-		unsigned int targinfosize,
 	);
