@@ -151,38 +151,39 @@ rawsnat_tg6_save(const void *entry, const struct xt_entry_target *target)
 	       info->mask);
 }
 
-static struct xtables_target rawsnat_tg4_reg = {
-	.version       = XTABLES_VERSION,
-	.name          = "RAWSNAT",
-	.revision      = 0,
-	.family        = NFPROTO_IPV4,
-	.size          = XT_ALIGN(sizeof(struct xt_rawnat_tginfo)),
-	.userspacesize = XT_ALIGN(sizeof(struct xt_rawnat_tginfo)),
-	.help          = rawsnat_tg_help,
-	.parse         = rawsnat_tg4_parse,
-	.final_check   = rawsnat_tg_check,
-	.print         = rawsnat_tg4_print,
-	.save          = rawsnat_tg4_save,
-	.extra_opts    = rawsnat_tg_opts,
-};
-
-static struct xtables_target rawsnat_tg6_reg = {
-	.version       = XTABLES_VERSION,
-	.name          = "RAWSNAT",
-	.revision      = 0,
-	.family        = NFPROTO_IPV6,
-	.size          = XT_ALIGN(sizeof(struct xt_rawnat_tginfo)),
-	.userspacesize = XT_ALIGN(sizeof(struct xt_rawnat_tginfo)),
-	.help          = rawsnat_tg_help,
-	.parse         = rawsnat_tg6_parse,
-	.final_check   = rawsnat_tg_check,
-	.print         = rawsnat_tg6_print,
-	.save          = rawsnat_tg6_save,
-	.extra_opts    = rawsnat_tg_opts,
+static struct xtables_target rawsnat_tg_reg[] = {
+	{
+		.version       = XTABLES_VERSION,
+		.name          = "RAWSNAT",
+		.revision      = 0,
+		.family        = NFPROTO_IPV4,
+		.size          = XT_ALIGN(sizeof(struct xt_rawnat_tginfo)),
+		.userspacesize = XT_ALIGN(sizeof(struct xt_rawnat_tginfo)),
+		.help          = rawsnat_tg_help,
+		.parse         = rawsnat_tg4_parse,
+		.final_check   = rawsnat_tg_check,
+		.print         = rawsnat_tg4_print,
+		.save          = rawsnat_tg4_save,
+		.extra_opts    = rawsnat_tg_opts,
+	},
+	{
+		.version       = XTABLES_VERSION,
+		.name          = "RAWSNAT",
+		.revision      = 0,
+		.family        = NFPROTO_IPV6,
+		.size          = XT_ALIGN(sizeof(struct xt_rawnat_tginfo)),
+		.userspacesize = XT_ALIGN(sizeof(struct xt_rawnat_tginfo)),
+		.help          = rawsnat_tg_help,
+		.parse         = rawsnat_tg6_parse,
+		.final_check   = rawsnat_tg_check,
+		.print         = rawsnat_tg6_print,
+		.save          = rawsnat_tg6_save,
+		.extra_opts    = rawsnat_tg_opts,
+	},
 };
 
 static void _init(void)
 {
-	xtables_register_target(&rawsnat_tg4_reg);
-	xtables_register_target(&rawsnat_tg6_reg);
+	xtables_register_targets(rawsnat_tg_reg,
+		sizeof(rawsnat_tg_reg) / sizeof(*rawsnat_tg_reg));
 }
